@@ -21,6 +21,7 @@ from unittest import mock
 from dogpile.cache import proxy
 from oslo_config import cfg
 from oslo_utils import uuidutils
+from pymemcache import KeepaliveOpts
 
 from oslo_cache import _opts
 from oslo_cache import core as cache
@@ -280,7 +281,7 @@ class CacheRegionTest(test_cache.BaseTestCase):
         self.config_fixture.config(group='cache',
                                    enabled=True,
                                    config_prefix='test_prefix',
-                                   backend='oslo_cache.dict',
+                                   backend='dogpile.cache.pymemcache',
                                    tls_cafile='path_to_ca_file',
                                    tls_keyfile='path_to_key_file',
                                    tls_certfile='path_to_cert_file',
@@ -298,7 +299,7 @@ class CacheRegionTest(test_cache.BaseTestCase):
         self.config_fixture.config(group='cache',
                                    enabled=True,
                                    config_prefix='test_prefix',
-                                   backend='oslo_cache.dict',
+                                   backend='dogpile.cache.pymemcache',
                                    tls_enabled=True)
 
         fake_context = mock.Mock()
@@ -322,7 +323,7 @@ class CacheRegionTest(test_cache.BaseTestCase):
         self.config_fixture.config(group='cache',
                                    enabled=True,
                                    config_prefix='test_prefix',
-                                   backend='oslo_cache.dict',
+                                   backend='dogpile.cache.pymemcache',
                                    tls_enabled=True,
                                    tls_cafile='path_to_ca_file',
                                    tls_keyfile='path_to_key_file',
@@ -352,6 +353,240 @@ class CacheRegionTest(test_cache.BaseTestCase):
                 config_dict['test_prefix.arguments.tls_context'],
             )
 
+    def test_cache_pymemcache_socket_kalive_enabled_with_wrong_backend(self):
+        """Validate we build a config without the retry option when retry
+        is disabled.
+        """
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='oslo_cache.dict',
+                                   enable_socket_keepalive=True)
+
+        self.assertRaises(
+            exception.ConfigurationError,
+            cache._build_cache_config,
+            self.config_fixture.conf
+        )
+
+    def test_cache_pymemcache_socket_keepalive_disabled(self):
+        """Validate we build a dogpile.cache dict config without keepalive."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   socket_keepalive_idle=2,
+                                   socket_keepalive_interval=2,
+                                   socket_keepalive_count=2)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertFalse(
+            self.config_fixture.conf.cache.enable_socket_keepalive)
+        self.assertNotIn(
+            'test_prefix.arguments.socket_keepalive', config_dict)
+
+    def test_cache_pymemcache_socket_keepalive_enabled(self):
+        """Validate we build a dogpile.cache dict config with keepalive."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   enable_socket_keepalive=True)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertTrue(
+            self.config_fixture.conf.cache.enable_socket_keepalive)
+
+        self.assertIsInstance(
+            config_dict['test_prefix.arguments.socket_keepalive'],
+            KeepaliveOpts
+        )
+
+    def test_cache_pymemcache_socket_keepalive_with_config(self):
+        """Validate we build a socket keepalive with the right config."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   enable_socket_keepalive=True,
+                                   socket_keepalive_idle=12,
+                                   socket_keepalive_interval=38,
+                                   socket_keepalive_count=42)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertTrue(
+            self.config_fixture.conf.cache.enable_socket_keepalive)
+
+        self.assertTrue(
+            config_dict['test_prefix.arguments.socket_keepalive'],
+            KeepaliveOpts
+        )
+        self.assertEqual(
+            12,
+            config_dict['test_prefix.arguments.socket_keepalive'].idle
+        )
+        self.assertEqual(
+            38,
+            config_dict['test_prefix.arguments.socket_keepalive'].intvl
+        )
+        self.assertEqual(
+            42,
+            config_dict['test_prefix.arguments.socket_keepalive'].cnt
+        )
+
+    def test_cache_pymemcache_retry_enabled_with_wrong_backend(self):
+        """Validate we build a config without the retry option when retry
+        is disabled.
+        """
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='oslo_cache.dict',
+                                   enable_retry_client=True,
+                                   retry_attempts=2,
+                                   retry_delay=2)
+
+        self.assertRaises(
+            exception.ConfigurationError,
+            cache._build_cache_config,
+            self.config_fixture.conf
+        )
+
+    def test_cache_pymemcache_retry_disabled(self):
+        """Validate we build a config without the retry option when retry
+        is disabled.
+        """
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   retry_attempts=2,
+                                   retry_delay=2)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        opts = ['enable_retry_client', 'retry_attempts', 'retry_delay']
+
+        for el in opts:
+            self.assertNotIn('test_prefix.arguments.{}'.format(el),
+                             config_dict)
+
+    def test_cache_pymemcache_retry_enabled(self):
+        """Validate we build a dogpile.cache dict config with retry."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   enable_retry_client=True)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        opts = ['enable_retry_client', 'retry_attempts', 'retry_delay']
+
+        for el in opts:
+            self.assertIn('test_prefix.arguments.{}'.format(el), config_dict)
+
+    def test_cache_pymemcache_retry_with_opts(self):
+        """Validate we build a valid config for the retry client."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   enable_retry_client=True,
+                                   retry_attempts=42,
+                                   retry_delay=42)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertTrue(
+            self.config_fixture.conf.cache.enable_retry_client)
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.retry_attempts'],
+            42
+        )
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.retry_delay'],
+            42
+        )
+
+    def test_cache_pymemcache_retry_with_extra_opts(self):
+        """Validate we build a valid config for the retry client."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='dogpile.cache.pymemcache',
+                                   enable_retry_client=True,
+                                   retry_attempts=42,
+                                   retry_delay=42,
+                                   hashclient_retry_attempts=100,
+                                   hashclient_retry_delay=100,
+                                   dead_timeout=100)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertTrue(
+            self.config_fixture.conf.cache.enable_retry_client)
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.retry_attempts'],
+            42
+        )
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.retry_delay'],
+            42
+        )
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.hashclient_retry_attempts'],
+            100
+        )
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.hashclient_retry_delay'],
+            100
+        )
+
+        self.assertEqual(
+            config_dict['test_prefix.arguments.dead_timeout'],
+            100
+        )
+
+    def test_cache_dictionary_config_builder_flush_on_reconnect_enabled(self):
+        """Validate we build a sane dogpile.cache dictionary config."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='oslo_cache.dict',
+                                   memcache_pool_flush_on_reconnect=True)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertTrue(self.config_fixture.conf.cache.
+                        memcache_pool_flush_on_reconnect)
+        self.assertTrue(config_dict['test_prefix.arguments'
+                                    '.pool_flush_on_reconnect'])
+
+    def test_cache_dictionary_config_builder_flush_on_reconnect_disabled(self):
+        """Validate we build a sane dogpile.cache dictionary config."""
+        self.config_fixture.config(group='cache',
+                                   enabled=True,
+                                   config_prefix='test_prefix',
+                                   backend='oslo_cache.dict',
+                                   memcache_pool_flush_on_reconnect=False)
+
+        config_dict = cache._build_cache_config(self.config_fixture.conf)
+
+        self.assertFalse(self.config_fixture.conf.cache.
+                         memcache_pool_flush_on_reconnect)
+        self.assertFalse(config_dict['test_prefix.arguments'
+                                     '.pool_flush_on_reconnect'])
+
     def test_cache_debug_proxy(self):
         single_value = 'Test Value'
         single_key = 'testkey'
